<?php

/**
 * Spreed WebRTC Plugin
 *
 * Integrates Spreed Web RTC into the Zarafa environment
 *
 */

include_once('class.webmeetingsauth.php');

class Pluginspreedwebrtc extends Plugin {
	/**
	 * Constructor
	 */
	function Pluginspreedwebrtc(){}

	/**
	 * Called to initialize the plugin and register for hooks.
	 */
	function init(){
		$this->registerHook('server.core.settings.init.before');
		$this->registerHook('server.index.login.success');
	}

	/**
	 *
	 * @param String $eventID Identifier of the hook
	 * @param Array $data Reference to the data of the triggered hook
	 */
	function execute($eventID, &$data){
		switch($eventID){
			case 'server.core.settings.init.before':
				$this->onBeforeSettingsInit($data);
				break;
			case 'server.index.login.success':
				if ($_GET && array_key_exists("noredirect", $_GET)) {
					if ($GLOBALS["mapisession"]->isLoggedOn()) {
						// Authenticated
						$version = trim(file_get_contents('version'));
						$data = array(
							"user" => array(
								"username" 	=> addslashes($GLOBALS["mapisession"]->getUserName()),
								"fullname" 	=> addslashes($GLOBALS["mapisession"]->getFullName()),
								"entryid" 	=> bin2hex($GLOBALS["mapisession"]->getUserEntryid()),
								"email_address" => addslashes($GLOBALS["mapisession"]->getEmailAddress()),
								"smtp_address" 	=> addslashes($GLOBALS["mapisession"]->getSMTPAddress()),
								"search_key" 	=> bin2hex($GLOBALS["mapisession"]->getSearchKey()),
								"sessionid" 	=> $GLOBALS["mapisession"]->getSessionID()
							),
							"version" => array(
								"webapp"	=> $version,
								"zcp"		=> phpversion('mapi'),
								"server"	=> DEBUG_SHOW_SERVER ? DEBUG_SERVER_ADDRESS : '',
								"svn"		=> DEBUG_LOADER === LOAD_SOURCE ? svnversion() : ''
							),
							"server" => array(
								"enable_plugins"		=> !!ENABLE_PLUGINS,
								"enable_advanced_settings"	=> !!ENABLE_ADVANCED_SETTINGS,
								"enable_large_files" => !!ENABLE_LARGEFILES,
								"max_attachments"		=> null,
								"max_attachment_size"		=> getMaxUploadSize(),
								"max_attachment_total_size"	=> null,
								"external_url" => EXTERNAL_WEBAPP_URI,
								"large_file_limit" => LARGEFILES_MIN_LIMIT,
								"freebusy_load_start_offset" => FREEBUSY_LOAD_START_OFFSET,
								"freebusy_load_end_offset" => FREEBUSY_LOAD_END_OFFSET
							)
						);
					} else {
						$data = array("authenticated" => 0);
					}
					echo json_encode(array("zarafa" => $data));
					exit;
				}
				break;
		}
	}

	/**
	 * Called when the core Settings class is initialized and ready to accept sysadmin default
	 * settings. Registers the sysadmin defaults for the example plugin.
	 * @param Array $data Reference to the data of the triggered hook
	 */
	function onBeforeSettingsInit(&$data){
		$data['settingsObj']->addSysAdminDefaults(Array(
			'zarafa' => Array(
				'v1' => Array(
					'plugins' => Array(
						'spreedwebrtc' => Array(
							'enable' => PLUGIN_SPREEDWEBRTC_USER_DEFAULT_ENABLE,
								'domain' => PLUGIN_SPREEDWEBRTC_SPREED_DOMAIN,
								'url' => PLUGIN_SPREEDWEBRTC_SPREED_URL,
								'debug' => PLUGIN_SPREEDWEBRTC_DEBUG,
								'autostart' => PLUGIN_SPREEDWEBRTC_AUTO_START
						)
					)
				)
			)
		));
	}
}
?>
